<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

class AdminSimpleExportController extends ModuleAdminController
{
    protected $_log_file;
    protected $_csv_file;
    protected $action;
    protected $delimiter = '$';
    protected $cell_limit = 30000;
    protected $cell_action = 0;
    protected $compatibility = 0;
    protected $price = 0;

    public $id_shop;
    public $id_lang;

    public function __construct()
    {
        parent::__construct();
        if (!$this->module->active) {
            die('Module disabled');
        }
        $this->_log_file = dirname(__FILE__) . '/progress.txt';
        $action = pSQL(Tools::getValue('action'));
        if ($action === 'GetProgress') {
            return $this->getProgress();
        }
        if ($action === 'DownloadFile') {
            $this->_csv_file = pSQL(Tools::getValue('csvFile'));
            return $this->downloadFile();
        }
        $this->id_shop = (int) Tools::getValue('shop');
        $this->id_lang = (int) Tools::getValue('lang');
        $this->cell_limit = (int) Tools::getValue('cell_limit');
        $this->cell_action = (int) Tools::getValue('cell_action');
        $this->compatibility = (int) Tools::getValue('compatibility');
        $this->price = (int) Tools::getValue('price');
        $this->delimiter = pSQL(Tools::getValue('delimiter'));

        if ($this->id_shop > 0 && $this->id_lang > 0) {
            if (in_array($action, $this->module->types)) {
                $this->action = 'export' . $action;
                return $this->preProcess();
            }
        }
        die('Invalid Request');
    }

    public function preProcess()
    {
        $this->removeLogFile();
        if ($this->_csv_file = $this->getCsvFile()) {
            $this->removeCsvFile();
        }
        $this->module->sendCallback();
        ob_start();
        ini_set('max_execution_time', 0);
        set_time_limit(0);
        $this->{$this->action}();
        ob_end_clean();
    }

    public function getProgress()
    {
        if (file_exists($this->_log_file)) {
            $content = file_get_contents($this->_log_file);
            $rows = explode("\n", $content);
            $continue = (end($rows) == 'End of export.') ? false : true;
            if (!$continue) {
                $this->removeLogFile();
                if ($this->_csv_file = $this->getCsvFile()) {
                    $link = $this->context->link->getAdminLink(
                        'AdminSimpleExport', true, [], ['csvFile' => basename($this->_csv_file), 'action' => 'DownloadFile']
                    );
                    die(json_encode(['data' => $content, 'continue' => $continue, 'link' => $link]));
                }
            } else {
                die(json_encode(['data' => $content, 'continue' => $continue]));
            }
        }
        die(json_encode(['data' => null, 'continue' => false]));
    }

    public function downloadFile()
    {
        if (!preg_match('/export_[a-z]+_[0-9-_]+.csv/', $this->_csv_file)) {
            die;
        }
        $file = dirname(__FILE__) . '/' . $this->_csv_file;
        if (file_exists($file)) {
            $fp = fopen($file, 'rb');
            header('Content-Type: application/csv');
            header('Content-Disposition: attachment; filename=' . $this->_csv_file);
            header('Pragma: no-cache');
            fpassthru($fp);
            fclose($fp);
            ignore_user_abort(1);
            $this->removeCsvFile();
            exit;
        }
    }

    private function getCsvFile()
    {
        if ($files = glob(dirname(__FILE__) . '/export_*.csv')) {
            if (!empty($files[0])) {
                return $this->_csv_file = $files[0];
            }
        }
        return $this->_csv_file = null;
    }

    public function exportProducts()
    {
        $start_time = microtime(true);
        $this->log('Starting...');
        $output = [];
        $get_product_list = Db::getInstance()->executeS(
            'SELECT `id_product` FROM `' . _DB_PREFIX_ . 'product` WHERE `id_shop_default` = ' . $this->id_shop
        );
        $this->log('Getting products list...');
        if (!$get_product_list) {
            $this->log('No products found, terminating...');
            $this->removeLogFile();
            die(json_encode(['data' => null, 'continue' => false]));
        }
        $id_product_list = array_map('intval', array_column($get_product_list, 'id_product'));
        $this->log('Found ' . count($id_product_list) . ' product(s), processing...');
        foreach ($id_product_list as $id_product) {
            $this->log('Processing ID: ' . $id_product . '...');
            $row = $this->getProduct((int) $id_product);
            $this->log('Finish ID: ' . $id_product . ' Name: ' . $row['Name*'] . '.');
            $output[] = $row;
        }
        if ($output) {
            $file_name = 'export_products_' . date('Y-m-d_H-i-s') . '.csv';
            $this->log('Writing CSV file: ' . $file_name . '...');
            $file_path = dirname(__FILE__) . '/' . $file_name;
            $file_open = fopen($file_path, 'w');
            fputs($file_open, implode($this->delimiter, array_keys($output[0])) . "\n");
            foreach ($output as &$line) {
                $line = str_replace($this->delimiter, '', $line);
                $line = str_replace("\n", '', $line);
                $line = str_replace("\r", '', $line);
                fputs($file_open, implode($this->delimiter, $line) . "\n");
            }
            fclose($file_open);
            $this->log('Writing CSV file: ' . $file_name . ' complete.');
        }
        $this->log('End of export.', true);
    }

    public function exportCategories()
    {        
        $start_time = microtime(true);
        $this->log('Starting...');
        $output = [];
        $get_category_list = Db::getInstance()->executeS(
            'SELECT `id_category` FROM `' . _DB_PREFIX_ . 'category` WHERE `id_shop_default` = ' . $this->id_shop
        );
        $this->log('Getting categories list...');
        if (!$get_category_list) {
            $this->log('No categories found, terminating...');
            $this->removeLogFile();
            die(json_encode(['data' => null, 'continue' => false]));
        }
        $id_category_list = array_map('intval', array_column($get_category_list, 'id_category'));
        $this->log('Found ' . count($id_category_list) . ' category(ies), processing...');
        foreach ($id_category_list as $id_category) {
            $this->log('Processing ID: ' . $id_category . '...');
            $row = $this->getCategory((int) $id_category);
            $this->log('Finish ID: ' . $id_category . ' Name: ' . $row['Name*'] . '.');
            $output[] = $row;
        }
        if ($output) {
            $file_name = 'export_categories_' . date('Y-m-d_H-i-s') . '.csv';
            $this->log('Writing CSV file: ' . $file_name . '...');
            $file_path = dirname(__FILE__) . '/' . $file_name;
            $file_open = fopen($file_path, 'w');
            fputs($file_open, implode($this->delimiter, array_keys($output[0])) . "\n");
            foreach ($output as &$line) {
                $line = str_replace($this->delimiter, '', $line);
                $line = str_replace("\n", '', $line);
                $line = str_replace("\r", '', $line);
                fputs($file_open, implode($this->delimiter, $line) . "\n");
            }
            fclose($file_open);
            $this->log('Writing CSV file: ' . $file_name . ' complete.');
        }
        $this->log('End of export.', true);
    }

    public function exportBrands()
    {
        $start_time = microtime(true);
        $this->log('Starting...');
        $output = [];
        $get_brand_list = Db::getInstance()->executeS(
            'SELECT m.`id_manufacturer` FROM `' . _DB_PREFIX_ . 'manufacturer` m
            JOIN `' . _DB_PREFIX_ . 'manufacturer_shop` s USING (`id_manufacturer`)
            WHERE s.`id_shop` = ' . $this->id_shop
        );
        $this->log('Getting brands list...');
        if (!$get_brand_list) {
            $this->log('No brands found, terminating...');
            $this->removeLogFile();
            die(json_encode(['data' => null, 'continue' => false]));
        }
        $id_brand_list = array_map('intval', array_column($get_brand_list, 'id_manufacturer'));
        $this->log('Found ' . count($id_brand_list) . ' brand(s), processing...');
        foreach ($id_brand_list as $id_manufacturer) {
            $this->log('Processing ID: ' . $id_manufacturer . '...');
            $row = $this->getBrand((int) $id_manufacturer);
            $this->log('Finish ID: ' . $id_manufacturer . ' Name: ' . $row['Name*'] . '.');
            $output[] = $row;
        }
        if ($output) {
            $file_name = 'export_brands_' . date('Y-m-d_H-i-s') . '.csv';
            $this->log('Writing CSV file: ' . $file_name . '...');
            $file_path = dirname(__FILE__) . '/' . $file_name;
            $file_open = fopen($file_path, 'w');
            fputs($file_open, implode($this->delimiter, array_keys($output[0])) . "\n");
            foreach ($output as &$line) {
                $line = str_replace($this->delimiter, '', $line);
                $line = str_replace("\n", '', $line);
                $line = str_replace("\r", '', $line);
                fputs($file_open, implode($this->delimiter, $line) . "\n");
            }
            fclose($file_open);
            $this->log('Writing CSV file: ' . $file_name . ' complete.');
        }
        $this->log('End of export.', true);
    }

    public function exportSuppliers()
    {
        $start_time = microtime(true);
        $this->log('Starting...');
        $output = [];
        $get_supplier_list = Db::getInstance()->executeS(
            'SELECT m.`id_supplier` FROM `' . _DB_PREFIX_ . 'supplier` m
            JOIN `' . _DB_PREFIX_ . 'supplier_shop` s USING (`id_supplier`)
            WHERE s.`id_shop` = ' . $this->id_shop
        );
        $this->log('Getting suppliers list...');
        if (!$get_supplier_list) {
            $this->log('No suppliers found, terminating...');
            $this->removeLogFile();
            die(json_encode(['data' => null, 'continue' => false]));
        }
        $id_supplier_list = array_map('intval', array_column($get_supplier_list, 'id_supplier'));
        $this->log('Found ' . count($id_supplier_list) . ' supplier(s), processing...');
        foreach ($id_supplier_list as $id_supplier) {
            $this->log('Processing ID: ' . $id_supplier . '...');
            $row = $this->getSupplier((int) $id_supplier);
            $this->log('Finish ID: ' . $id_supplier . ' Name: ' . $row['Name*'] . '.');
            $output[] = $row;
        }
        if ($output) {
            $file_name = 'export_suppliers_' . date('Y-m-d_H-i-s') . '.csv';
            $this->log('Writing CSV file: ' . $file_name . '...');
            $file_path = dirname(__FILE__) . '/' . $file_name;
            $file_open = fopen($file_path, 'w');
            fputs($file_open, implode($this->delimiter, array_keys($output[0])) . "\n");
            foreach ($output as &$line) {
                $line = str_replace($this->delimiter, '', $line);
                $line = str_replace("\n", '', $line);
                $line = str_replace("\r", '', $line);
                fputs($file_open, implode($this->delimiter, $line) . "\n");
            }
            fclose($file_open);
            $this->log('Writing CSV file: ' . $file_name . ' complete.');
        }
        $this->log('End of export.', true);
    }

    public function getProduct($id_product)
    {
        $ret = [];
        $product = new Product($id_product, true, $this->id_lang, $this->id_shop);
        $ret['ID'] = (string) $product->id;
        $ret['Active (0/1)'] = $product->active;
        $ret['Name*'] = $product->name;
        $categories = $product->getProductCategories($id_product);
        if (($key = array_search($product->id_category_default, $categories)) !== false) {
            unset($categories[$key]);
        }
        array_unshift($categories, $product->id_category_default);
        $ret['Categories (x,y,z...)'] = implode(',', $categories);
        if (!$this->price) {
            $ret['Price tax excluded'] = (string) $product->price;
        } else {
            $ret['Price tax included'] = (string) $product->getPublicPrice();
        }
        $ret['Tax rule ID'] = $product->id_tax_rules_group;
        $ret['Cost price'] = $product->wholesale_price;
        $ret['On sale (0/1)'] = $product->on_sale;
        $Discount_amount = '';
        $Discount_percent = '';
        $Discount_from = '';
        $Discount_to = '';
        if (!empty($product->specificPrice)) {
            if (!empty($product->specificPrice['reduction_type'])) {
                if ($product->specificPrice['reduction_type'] == 'percentage') {
                    if (!empty($product->specificPrice['reduction'])) {
                        $Discount_percent = (string) intval($product->specificPrice['reduction'] * 100);
                    }
                } elseif ($product->specificPrice['reduction_type'] == 'amount') {
                    if (!empty($product->specificPrice['reduction'])) {
                        $Discount_amount = (string) $product->specificPrice['reduction'];
                    }
                }
                $Discount_from = $product->specificPrice['from'] ?? '';
                $Discount_to = $product->specificPrice['to'] ?? '';
            }
        }
        $ret['Discount amount'] = $Discount_amount;
        $ret['Discount percent'] = $Discount_percent;
        $ret['Discount from (yyyy-mm-dd)'] = $Discount_from;
        $ret['Discount to (yyyy-mm-dd)'] = $Discount_to;
        $ret['Reference #'] = $product->reference;
        $ret['Supplier reference #'] = $product->supplier_reference;
        $ret['Supplier'] = $product->supplier_name;
        $ret['Brand'] = $product->manufacturer_name;
        $ret['EAN13'] = $product->ean13;
        $ret['UPC'] = $product->upc;
        if (!$this->compatibility) {
            $ret['MPN'] = $product->mpn;
        }
        $ret['Ecotax'] = $product->ecotax;
        $ret['Width'] = $product->width;
        $ret['Height'] = $product->height;
        $ret['Depth'] = $product->depth;
        $ret['Weight'] = $product->weight;
        if (!$this->compatibility) {
            $ret['Delivery time of in-stock products:'] = $product->delivery_in_stock;
            $ret['Delivery time of out-of-stock products with allowed orders:'] = $product->delivery_out_stock;
        }
        $ret['Quantity'] = $product->quantity;
        $ret['Minimal quantity'] = $product->minimal_quantity;
        if (!$this->compatibility) {
            $ret['Low stock level'] = $product->low_stock_threshold;
            $ret['Receive a low stock alert by email'] = $product->low_stock_alert;
        }
        $ret['Visibility'] = $product->visibility;
        $ret['Additional shipping cost'] = $product->additional_shipping_cost;
        $ret['Unit for base price'] = $product->unity;
        $ret['Base price'] = $product->unit_price;
        $ret['Summary'] = $product->description_short;
        $description = $product->description;
        if ($this->cell_limit && (mb_strlen($product->description) > $this->cell_limit)) {
            $description = '';
            if ($this->cell_action == 1) {
                $description = mb_substr($product->description, 0, $this->cell_limit);
            }
        }
        $ret['Description'] = $description;
        $tags = '';
        if (!empty($product->tags)) {
            if (!empty($product->tags[$this->id_lang])) {
                $tags = implode(',', $product->tags[$this->id_lang]);
            }
        }
        $ret['Tags (x,y,z...)'] = $tags;
        $ret['Meta title'] = $product->meta_title;
        $ret['Meta keywords'] = $product->meta_keywords;
        $ret['Meta description'] = $product->meta_description;
        $ret['Rewritten URL'] = $product->link_rewrite;
        $ret['Label when in stock'] = $product->available_now;
        $ret['Label when backorder allowed'] = $product->available_later;
        $ret['Available for order (0 = No, 1 = Yes)'] = $product->available_for_order;
        $ret['Product availability date'] = $product->available_date;
        $ret['Product creation date'] = $product->date_add;
        $ret['Show price (0 = No, 1 = Yes)'] = $product->show_price;
        $images_links = '';
        $images_alts = '';
        if ($images_product = $product->getImages($this->id_lang)) {
            foreach ($images_product as $k => $i) {
                $image = $this->context->link->getImageLink(
                    $product->link_rewrite, (int) $i['id_image'], null
                );
                if ($image) {
                    $images_product[$k]['link'] = $image;
                }
            }
            $images_links = implode(',', array_column($images_product, 'link'));
            $images_alts = implode(',', array_column($images_product, 'legend'));
        }
        $ret['Image URLs (x,y,z...)'] = $images_links;
        $ret['Image alt texts (x,y,z...)'] = $images_alts;
        $ret['Delete existing images (0 = No, 1 = Yes)'] = '0';
        $features = '';
        if ($features_product = $product->getFeatures()) {
            $features_array = [];
            foreach ($features_product as $k => $f) {
                $feature = Feature::getFeature($this->id_lang, $f['id_feature']);
                $lang_values = FeatureValue::getFeatureValueLang($f['id_feature_value']);
                $value = FeatureValue::selectLang($lang_values, $this->id_lang);
                if (!$this->compatibility) {
                    $features_array[] = $feature['name'] . ':' . $value . ':' . (string) $k +1  . ':' . $f['custom'];
                } else {
                    $features_array[] = $feature['name'] . ':' . $value . ':' . (string) $k + 1 . ':' . $feature['name'];
                }
            }
            $features = implode(',', $features_array);
        }
        $ret['Feature (Name:Value:Position:Customized)'] = $features;
        $ret['Available online only (0 = No, 1 = Yes)'] = $product->online_only;
        $ret['Condition'] = $product->condition;
        $ret['Customizable (0 = No, 1 = Yes)'] = $product->customizable;
        $ret['Uploadable files (0 = No, 1 = Yes)'] = $product->uploadable_files;
        $ret['Text fields (0 = No, 1 = Yes)'] = $product->text_fields;
        $ret['Action when out of stock'] = $product->out_of_stock;
        $ret['Virtual product (0 = No, 1 = Yes)'] = $product->is_virtual;
        $ret['File URL'] = '';
        $ret['Number of allowed downloads'] = '0';
        $ret['Expiration date (yyyy-mm-dd)'] = '0000-00-00';
        $ret['Number of days'] = '0';
        $ret['ID / Name of the store'] = $product->id_shop_default;
        $ret['Advanced stock management'] = $product->advanced_stock_management;
        $ret['Depends on stock'] = $product->depends_on_stock ?? 0;
        $ret['Warehouse'] = 0;
        $accessories = '';
        if ($accessories_product = $product->getAccessoriesLight($this->id_lang, $product->id)) {
            $accessories = implode(',', array_column($accessories_product, 'reference'));
        }
        $ret['Accessories (x,y,z...)'] = $accessories;
        unset($product);
        return $ret;
    }

    public function getCategory($id_category)
    {
        $ret = [];
        $category = new Category($id_category, $this->id_lang, $this->id_shop);
        $parent_name = '';
        if ($category->id_parent) {
            $parent_category = new Category($category->id_parent, $this->id_lang, $this->id_shop);
            $parent_name = $parent_category->name;
            unset($parent_category);
        }
        $ret['ID'] = (string) $category->id;
        $ret['Active (0/1)'] = $category->active;
        $ret['Name*'] = $category->name;
        $ret['Parent category'] = $parent_name;
        $ret['Root category (0/1)'] = $category->is_root_category;
        $description = $category->description;
        if ($this->cell_limit && (mb_strlen($category->description) > $this->cell_limit)) {
            $description = '';
            if ($this->cell_action == 1) {
                $description = mb_substr($category->description, 0, $this->cell_limit);
            }
        }
        $ret['Description'] = $description;
        $ret['Meta title'] = $category->meta_title;
        $ret['Meta keywords'] = $category->meta_keywords;
        $ret['Meta description'] = $category->meta_description;
        $ret['Rewritten URL'] = $category->link_rewrite;
        $image = '';
        if ($category->id_image) {
            $image = $this->context->link->getCatImageLink($category->link_rewrite, (int) $category->id_image, null);
        }
        $ret['Image URL'] = $image;
        $ret['ID / Name of the store'] = $category->id_shop_default;
        unset($category);
        return $ret;
    }

    public function getBrand($id_brand)
    {
        $ret = [];
        $brand = new Manufacturer($id_brand, $this->id_lang);
        $ret['ID'] = (string) $brand->id;
        $ret['Active (0/1)'] = $brand->active;
        $ret['Name*'] = $brand->name;
        $description = $brand->description;
        if ($this->cell_limit && (mb_strlen($brand->description) > $this->cell_limit)) {
            $description = '';
            if ($this->cell_action == 1) {
                $description = mb_substr($brand->description, 0, $this->cell_limit);
            }
        }
        $ret['Description'] = $description;
        $ret['Meta title'] = $brand->meta_title;
        $ret['Meta keywords'] = $brand->meta_keywords;
        $ret['Meta description'] = $brand->meta_description;
        $image = $this->context->link->getManufacturerImageLink($brand->id);
        $ret['Image URL'] = $image ? $image : '';
        $ret['ID / Name of the store'] = $this->id_shop;
        unset($brand);
        return $ret;
    }

    public function getSupplier($id_supplier)
    {
        $ret = [];
        $supplier = new Supplier($id_supplier, $this->id_lang);
        $ret['ID'] = (string) $supplier->id;
        $ret['Active (0/1)'] = $supplier->active;
        $ret['Name*'] = $supplier->name;
        $description = $supplier->description;
        if ($this->cell_limit && (mb_strlen($supplier->description) > $this->cell_limit)) {
            $description = '';
            if ($this->cell_action == 1) {
                $description = mb_substr($supplier->description, 0, $this->cell_limit);
            }
        }
        $ret['Description'] = $description;
        $ret['Meta title'] = $supplier->meta_title;
        $ret['Meta keywords'] = $supplier->meta_keywords;
        $ret['Meta description'] = $supplier->meta_description;
        $image = $this->context->link->getSupplierImageLink($supplier->id);
        $ret['Image URL'] = $image ? $image : '';
        $ret['ID / Name of the store'] = $this->id_shop;
        unset($supplier);
        return $ret;
    }

    public function log($string, $end = false)
    {
        $eol = $end ? '' : PHP_EOL;
        file_put_contents($this->_log_file, $string . $eol, FILE_APPEND);
    }

    public function removeLogFile()
    {
        if (file_exists($this->_log_file)) {
            unlink($this->_log_file);
        }
    }

    public function removeCsvFile()
    {
        if ($files = glob(dirname(__FILE__) . '/export_*.csv')) {
            foreach ($files as $file) {
                unlink($file);
            }
        }
        return $this->_csv_file = null;
    }
}
