<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class SimpleExport extends Module
{
    public $types = [
        'Products',
        'Categories',
        'Brands',
        'Suppliers'
    ];

    public function __construct()
    {
        $this->name = 'simpleexport';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'metacreo';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->ps_versions_compliancy = ['min' => '1.7.8.3', 'max' => _PS_VERSION_];
        parent::__construct();
        $this->displayName = 'SimpleExport';
        $this->description = 'Prestashop data export module. Donations are <a href="https://www.paypal.com/paypalme/NewestVision" target="_blank">welcome</a>.';
    }

    public function install()
    {
        return parent::install() && $this->installTab() && $this->registerHook('actionAdminControllerSetMedia');
    }

    public function uninstall()
    {
        return parent::uninstall() && $this->uninstallTab() && $this->unregisterHook('actionAdminControllerSetMedia');
    }

    public function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->name = [];
        $tab->class_name = 'AdminSimpleExport';
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Simple Export';
        }
        $tab->id_parent = -1;
        $tab->module = $this->name;
        return $tab->add();
    }

    public function uninstallTab()
    {
        $id_tab = (int) Tab::getIdFromClassName('AdminSimpleExport');
        if ($id_tab) {
            $tab = new Tab($id_tab);
            return $tab->delete();
        }
        return false;
    }

    public function hookActionAdminControllerSetMedia()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addJS(_MODULE_DIR_ . $this->name . '/views/js/back.js');
        Media::addJsDef([
            $this->name . '_urladmin' => $this->context->link->getAdminLink('AdminSimpleExport'),
            $this->name . '_run' => false,
            $this->name . '_action' => null,
            $this->name . '_lang' => null,
            $this->name . '_shop' => null,
            $this->name . '_cell_limit' => null,
            $this->name . '_cell_action' => null,
            $this->name . '_delimiter' => null,
            $this->name . '_compatibility' => null,
            $this->name . '_price' => null
        ]);
    }

    public function getContent()
    {
        if (!$this->active) {
            die('Module disabled');
        }
        $this->context->smarty->assign([
            $this->name . '_module_dir' => $this->_path,
            $this->name . '_types' => $this->types,
            $this->name . '_action' => null,
            $this->name . '_langs' => Language::getLanguages(false),
            $this->name . '_shops' => Shop::getShops(false),
            $this->name . '_lang' => null,
            $this->name . '_shop' => null,
            $this->name . '_cell_limit' => null,
            $this->name . '_cell_action' => null,
            $this->name . '_delimiter' => null,
            $this->name . '_compatibility' => null,
            $this->name . '_price' => null
        ]);
        $output = '';
        if (Tools::isSubmit($this->name)) {
            $type = Tools::getValue($this->name . '_action');
            $lang = (int) Tools::getValue($this->name . '_lang');
            $shop = (int) Tools::getValue($this->name . '_shop');
            $cell_limit = (int) Tools::getValue($this->name . '_cell_limit');
            $cell_action = (int) Tools::getValue($this->name . '_cell_action');
            $delimiter = pSQL(Tools::getValue($this->name . '_delimiter'));
            $compatibility = (int) Tools::getValue($this->name . '_compatibility');
            $price = (int) Tools::getValue($this->name . '_price');
            if (in_array($type, $this->types)) {
                $output .= $this->displayConfirmation('Processing ' . $type . '...');
                Media::addJsDef([
                    $this->name . '_run' => true,
                    $this->name . '_action' => $type,
                    $this->name . '_lang' => $lang,
                    $this->name . '_shop' => $shop,
                    $this->name . '_cell_limit' => $cell_limit,
                    $this->name . '_cell_action' => $cell_action,
                    $this->name . '_delimiter' => $delimiter,
                    $this->name . '_compatibility' => $compatibility,
                    $this->name . '_price' => $price
                ]);
                $this->context->smarty->assign([
                    $this->name . '_action' => $type,
                    $this->name . '_lang' => $lang,
                    $this->name . '_shop' => $shop,
                    $this->name . '_cell_limit' => $cell_limit,
                    $this->name . '_cell_action' => $cell_action,
                    $this->name . '_delimiter' => $delimiter,
                    $this->name . '_compatibility' => $compatibility,
                    $this->name . '_price' => $price
                ]);
            }
        }
        return $output .= $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
    }

    public function sendCallback()
    {
        ignore_user_abort(true);
        ini_set('memory_limit', -1);
        ini_set('max_execution_time', 0);
        set_time_limit(0);
        ob_start();
        echo $this->name.'_prestashop_' . getmypid();
        header('Connection: close');
        header('Content-Length: ' . ob_get_length());
        ob_end_flush();
        ob_flush();
        flush();
        if (function_exists('fastcgi_finish_request')) {
            fastcgi_finish_request();
        }
    }

}
